/*
TASK: BURN
LANG: C++
*/

#include <cmath>
#include <queue>
#include <cfloat>
#include <cstdio>
#include <vector>
#include <climits>

class edge
{
public:
    edge() {}
    edge(const unsigned int& f, const unsigned int& t, const unsigned int& l) : from(f), to(t), len(l) {}
    
    unsigned int from;
    unsigned int to;
    unsigned int len;
};

inline bool operator< (const edge& a, const edge& b)
{
    return a.len > b.len;
}

int main()
{
    unsigned int v, e;
    std::scanf("%u %u", &v, &e);
    std::vector<std::vector<edge> > graph(v);
    unsigned int x, y, z;
    std::vector<edge> edges;
    std::vector<unsigned int> powers(v, 0);
    for (unsigned int i = 0; i < e; i++)
    {
	std::scanf("%u %u %u", &x, &y, &z); x--; y--;
	graph[x].push_back(edge(x, y, z));
	graph[y].push_back(edge(y, x, z));
	edges.push_back(edge(x, y, z));
	powers[x]++; powers[y]++;
    }

    double best = DBL_MAX;
    for (unsigned int i = 0; i < v; i++)
    {
	std::priority_queue<edge> q;
	std::vector<unsigned int> burn_start(v, UINT_MAX);
        q.push(edge(UINT_MAX, i, 0));

	while (!q.empty())
	{
	    edge e;
	    do {e = q.top(); q.pop();} while (!q.empty() && burn_start[e.to] != UINT_MAX);
	    if (burn_start[e.to] != UINT_MAX) break;

	    burn_start[e.to] = e.len;
	    for (unsigned int j = 0; j < graph[e.to].size(); j++)
		if (burn_start[graph[e.to][j].to] == UINT_MAX)
		    q.push(edge(e.to, graph[e.to][j].to, graph[e.to][j].len + e.len + powers[e.to]));
	}

	unsigned int max = 0;
	for (unsigned int j = 0; j < v; j++)
	    if (burn_start[j] + powers[j] > max)
		max = burn_start[j] + powers[j];

	/*for (unsigned int i = 0; i < v; i++)
	    std::printf("%d ", burn_start[i]);
	std::printf("\n");*/

	if ((double)max >= best) continue;
	
	double d, curr;
	for (unsigned int j = 0; j < e; j++)
	{
	    edge& e = edges[j];
	    if (burn_start[e.from]+powers[e.from] > burn_start[e.to]+powers[e.to])
		e.from ^= e.to ^= e.from ^= e.to;
	    d = burn_start[e.from]+powers[e.from];
	    d += (
		 burn_start[e.from]+powers[e.from]+e.len <= burn_start[e.to]+powers[e.to] ? e.len :
		 burn_start[e.to]+powers[e.to]-burn_start[e.from]-powers[e.from] 
		 + (e.len-burn_start[e.to]-powers[e.to]+burn_start[e.from]+powers[e.from])/2.0
		 );

	    if (d > curr)
		curr = d;
	}

	if (curr < best)
	    best = curr;
    }

    std::printf("%.1f\n", best);
    
    return 0;
}

